@@ -23,9 +23,9 @@ proc tkeditpr_process_args {} {
             switch -exact -- $opt -server {
                 set TkGnats(ServerInfo) $val
             } -categories {
-                set TkGnats(CategoryList) $val
+                set TkGnats(CategoryFile) $val
             } -submitters {
-                set TkGnats(SubmitterList) $val
+                set TkGnats(SubmitterFile) $val
             } -responsible {
                 set TkGnats(ResponsibleFile) $val
             } -classes {
@@ -178,7 +178,7 @@ proc real_file_report {} {
         if {[check_suppressed_field $f]} {
             continue
         }
-	if {![check_mandatory_field $f]} {
+	if {$f != ">Responsible" && ![check_mandatory_field $f]} {
 	    continue
 	}
         set  val [string trim $frm($f) " \n\t\f!:~-#_?"]
@@ -287,7 +287,6 @@ proc real_file_report {} {
     set done 0
     while {!$done} {
         set stat [real_file_report_$TkGnats(GNATS_ACCESS_METHOD) errs]
-        #puts "stat=$stat errs=$errs"
         if {$stat == 0} {
             set done 1
         } {
@@ -864,7 +863,7 @@ proc edit_window {} {
     message .action.msg -aspect 10000  -relief sunken -bd 1 -text ""
     pack    .action.msg -side left -fill x -expand 1
 
-    # Get the maximum width of the value fields for the bagged_radiobar
+    # Get the maximum width of the value fields for the radiobar
     set Tkeditpr(value_width) 0
     foreach tag $Tkeditpr(radioflds) {
         set f [ftrim $flds($tag)]
@@ -894,8 +893,11 @@ proc edit_window {} {
     radiobar_frame .eflds .eflds.lb
     foreach {efield elist} [list Class $TkGnats(ClassesList) State $TkGnats(StatesList) Priority {low medium high} Severity {non-critical serious critical} Confidential {no yes}] {
         if {[check_suppressed_field $efield] != 1} {
-            bagged_radiobar .eflds.lb [string tolower $efield] $efield \
-                    $elist None frm $Tkeditpr(value_width)
+            if {![info exists TkGnats(${efield}PerRow)]} {
+                set TkGnats(${efield}PerRow) [llength $elist]
+            }
+            radiobar .eflds.lb [string tolower $efield] $efield \
+                    $elist $TkGnats(${efield}PerRow) None > frm $Tkeditpr(value_width)
         }
     }
     pack .eflds.lb -side left -pady 0
@@ -933,7 +935,16 @@ proc edit_window {} {
     fillfrm
 
     tkwait visibility .
-    
+
+    # If a listbox field value is no longer valid (eg a responsible who isn't), a blank value
+    # is placed in the entry field. Warn the user.
+    foreach f $Tkeditpr(listboxflds) {
+        set  val $frm($f)
+        if {$val == ""} {
+            Msg "The value for '[get_field_alias $f]' is no longer valid. Please choose a new value from the list."
+        }
+    }
+        
     if {"$missing_list" != ""} {
         Msg "The following fields were missing from the report but will be added when you Save Changes:\n" [join $missing_list \n]
         set Tkeditpr(parsed_flds) [concat $Tkeditpr(parsed_flds) $missing_list]
@@ -951,12 +962,12 @@ foreach f { tkpr_library.tcl tkprhelp.tcl tkprfont.tcl tkprdatesel.tcl } {
     source $TkGnats(lib)/$f
 }
 
-# If TkGnats(GNATS_ACCESS_METHOD) != "socket" then this does nothing
-if {[open_socket_gnatsd 1] == "-1"} {
+if {[get_gnats_config] == "-1"} {
     exit
 }
 
-if {[get_gnats_config] == "-1"} {
+# If TkGnats(GNATS_ACCESS_METHOD) != "socket" then this does nothing
+if {[open_socket_gnatsd 1] == "-1"} {
     exit
 }
 
