@@ -4,7 +4,7 @@ proc tksendpr_usage {{str ""}} {
     tk_dialog .tkgnatsError "TkGnats Error" \
             "${str}usage: tksendpr -server 'ServerInfo' \[-categories 'list'\] \
              \[-classes 'list'\] \[-states 'list'\] \
-            \[-db 'database'\] \[-submitters 'list'\] \[-responsible 'list'\]" "error" 0 "OK"
+             \[-submitters 'list'\] \[-responsible 'list'\]" "error" 0 "OK"
     exit
 }
 
@@ -25,9 +25,9 @@ proc tksendpr_process_args {} {
 	switch -exact -- $opt -server {
             set TkGnats(ServerInfo) $val
         } -categories {
-	    set TkGnats(CategoryList)  $val
+	    set TkGnats(CategoryFile)  $val
 	} -submitters {
-	    set TkGnats(SubmitterList) $val
+	    set TkGnats(SubmitterFile) $val
         } -responsible {
             set TkGnats(ResponsibleFile) $val
         } -classes {
@@ -535,8 +535,12 @@ frame .eflds
 radiobar_frame .eflds .eflds.lb
 foreach {efield elist} [list Class $TkGnats(ClassesList) State $TkGnats(StatesList) Priority {low medium high} Severity {non-critical serious critical} Confidential {no yes}] {
     if {![check_suppressed_field $efield]} {
-        radiobar     .eflds.lb [string tolower $efield] $efield $elist None
-        radiobar_set .eflds.lb [string tolower $efield] $TkGnats(CreateDefault$efield)
+        if {![info exists TkGnats(${efield}PerRow)]} {
+            set TkGnats(${efield}PerRow) [llength $elist]
+        }
+        radiobar     .eflds.lb [string tolower $efield] $efield $elist $TkGnats(${efield}PerRow) None
+        radiobar_set .eflds.lb [string tolower $efield] $TkGnats(CreateDefault$efield) \
+                $elist $TkGnats(${efield}PerRow)
     } {
         set $efield [lindex $elist 0]
     }
