@@ -22,6 +22,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #include "gnats.h"
 #include "gnatsd.h"
 #include "query.h"
+#include "db_conf.h"
 
 char *current_pr = (char *)NULL;
 
@@ -34,8 +35,12 @@ char *current_pr = (char *)NULL;
 #endif
 
 extern char myname[MAXHOSTNAMELEN];
+extern int require_db;
 extern int searching;
+extern int doret;
+extern int doescape;
 extern Index *search;
+extern char *index_filename;
 
 #ifdef HAVE_KERBEROS
 extern int client_authorized;
@@ -65,72 +70,11 @@ noargs (s)
   printf ("%d %s takes no arguments.\r\n", CODE_ERROR, s);
 }
 
-char *
-lookup_pr_path (p)
-     char *p;
-{
-  char *path;
-  FILE *fp;
-
-  /* They gave it to us with the category (possibly).  */
-  if (((char *) strchr (p, '/')) == NULL)
-    path = get_category (p);
-  else
-    {
-      path = (char *) xmalloc (PATH_MAX);
-      sprintf (path, "%s/%s", gnats_root, p);
-    }
-  fp = fopen (path, "r");
-  if (fp == (FILE *)NULL)
-    {
-      printf ("%d Invalid PR %s.\r\n", CODE_INVALID_PR, p);
-      return NULL;
-    }
-
-  fclose (fp);
-  return path;
-}
-
-int
-qualify (ac, av)
-     int ac;
-     char **av;
-{
-  char pat[40];
-  int i;
-  Index *j;
-  int found = 0;
-
-  for (i = 0; i < ac; i++)
-    {
-      strcpy (pat, av[i]);
-      strcat (pat, "\\'");
-      for (j = index_chain ; j ; j = j->next)
-	if (regcmp (pat, j->number) == 0)
-	  {
-	    if (searching && !pr_matches (search, j))
-	      {
-		av[i] = (char *)NULL;
-		break;
-	      }
-	    else
-	      {
-		char *path = (char *) xmalloc (PATH_MAX);
-		sprintf (path, "%s/%s/%s", gnats_root, j->category, j->number);
-		av[i] = path;
-		found = 1;
-	      }
-	  }
-    }
-
-  return found;
-}
-
 int
 addr_cmp (a, b)
-     char **a, **b;
+     const void *a, *b;
 {
-  return strcmp (*a, *b);
+     return strcmp (*(char **)a, *(char **)b);
 }
 
 /* Given an array of possible addresses, make them into a real list with
@@ -146,6 +90,7 @@ unify_addrs (addrs, naddrs)
   int i, j = 0;
 
   qsort ((char *)addrs, naddrs, sizeof (char *), addr_cmp);
+    
   if (naddrs == 1)
     return 0;
 
@@ -175,6 +120,7 @@ get_text ()
 #endif
   char *buf, *tmpdir;
   MsgType r;
+  int i;
 
   tmpdir = getenv ("TMPDIR");
   if (tmpdir == NULL)
@@ -210,9 +156,13 @@ get_text ()
 	case PR_ok:
 	  {
 	    register char *bp = buf;
-	    if (buf[0] == '.' && (buf[1] == '\n' || buf[1] == '\0'))
+	    if (buf[0] == '.' && (buf[1] == '\r' ||buf[1] == '\n' || buf[1] == '\0'))
 	      goto done;
-	    fputs (bp, tf);
+            /* Remove escape character added by write_multiline() */
+            i = 0;
+            if (bp[0] == '.')
+              i = 1;
+	    fputs (bp + i, tf);
 	    fputs ("\n", tf);
 	    if (ferror (tf))
 	      goto die; /* XXX */
@@ -248,53 +198,13 @@ get_text ()
   return NULL;
 }
 
-void
-daemon_pr (p, print)
-     char *p;
-     int print;
-{
-  char *path;
-
-  if (p[0] != '/')
-    path = lookup_pr_path (p);
-  else
-    {
-      path = p;
-      p = NULL;
-    }
-
-  if (path)
-    {
-      if (get_pr (path, p, quiet))
-	{
-	  if (print)
-	    print_pr (path, p, 1);
-	  else
-	    {
-	      /* Even though we shouldn't do multiple loads of different
-		 PRs in one session, let's be safe.  */
-	      if (current_pr)
-		xfree (current_pr);
-	      current_pr = path;
-	    }
-	}
-      else
-	printf ("%d-Invalid PR %s.\r\n", CODE_INVALID_PR, p);
-
-      if (print)
-	xfree (path);
-    }
-  else
-    /* We should have always verified each PR.  */
-    abort ();
-}
-
 void
 do_query (ac, av)
      int ac;
      char **av;
 {
-  int i;
+  char pat[40], *p, *n;
+  int i, opened;
   Index *j;
   int hit = 0;
 
@@ -305,36 +215,39 @@ do_query (ac, av)
       /* We weren't given a list of PRs to check, so we do the
 	 whole shooting match.  */
       for (j = index_chain; j ; j = j->next)
-	if (pr_matches (search, j))
+	if (pr_matches (search, j, &opened) && do_pr (j, opened))
 	  {
-	    if (do_pr (j))
-	      {
-		hit = 1;
-		printf ("\r\n");
-	      }
+	    hit = 1;
+	    printf ("\r\n");
 	  }
-      if (! hit)
-	printf ("%d No PRs match.\r\n", CODE_NO_PRS);
-    }
-  else if (qualify (ac, av))
-    {
-      for (i = 0; i < ac; i++)
-	{
-	  /* Note we may be clearing it out inside qualify(), if they
-	     gave a search qualifier that cancels this particular PR out.  */
-	  if (av[i])
-	    {
-	      daemon_pr (av[i], 1);
-	      hit = 1;
-	      if (! (query_format & FORMAT_FULL))
-		printf ("\r\n");
-	    }
-	}
     }
   else
+    {
+      for (i = 0; i < ac; i++)
+        {
+          p = av[i];
+          if ((n = (char *) strchr (p, '/')) != NULL) /* Remove the category */
+               p = ++n;
+          strcpy (pat, p);
+          strcat (pat, "\\'");
+          for (j = index_chain ; j ; j = j->next)
+       	    if (regcmp (pat, j->number) == 0)
+              {
+		if (pr_matches (search, j, &opened) && do_pr (j, opened))
+                  {
+                    hit = 1;
+                    printf ("\r\n");
+                  }
+                /* Only go thru for a particular pattern until we make a
+                   match; once we do, move on to the next argument.  */
+                break;
+              }
+        }
+    }
+  
+  if (! hit)
     printf ("%d No PRs match.\r\n", CODE_NO_PRS);
-
-  if (hit)
+  else
     printf (".\r\n");
 }
 
@@ -388,7 +301,9 @@ GNATS_full (ac, av)
      char **av;
 {
   query_format = FORMAT_FULL;
+  doescape = 1;
   do_query (ac, av);
+  doescape = 0;
 }
 
 void
@@ -400,26 +315,19 @@ GNATS_sqlf (ac, av)
   do_query (ac, av);
 }
 
+void
+GNATS_sql2 (ac, av)
+     int ac;
+     char **av;
+{
+  query_format = FORMAT_SQL2;
+  do_query (ac, av);
+}
+
 /* Used in edit.c to force creation of a PR's text.  We'll never do that
    from here.  */
 int force = 0;
 
-char *
-daemon_lock_pr (p, w, processid)
-     char *p, *w, *processid;
-{
-  char *path;
-
-  path = lookup_pr_path (p);
-  if (! path)
-    return NULL;
-
-  if (! lock_pr (path, w, processid))
-    return NULL;
-
-  return path;
-}
-
 void
 GNATS_lock (ac, av)
      int ac;
@@ -435,6 +343,10 @@ GNATS_lock (ac, av)
       return;
     }
 
+  path = lookup_pr_path (av[0]);
+  if (! path)
+    return;
+
   if (ac > 2)
     {
       /* XXX FIXME -- we need a cleaner approach to this.  */
@@ -448,15 +360,12 @@ GNATS_lock (ac, av)
 	  strcat (l, av[i]);
 	}
     }
-  path = daemon_lock_pr (av[0], av[1], l);
+
+  lock_pr (path, av[1], l);
+
   if (l)
     xfree (l);
-  if (! path)
-    return;
-
-  query_format = FORMAT_FULL;
-  daemon_pr (path, 1);
-  printf (".\r\n");
+  xfree (path);
 }
 
 void
@@ -469,29 +378,31 @@ GNATS_unlk (ac, av)
   if (! chk_nargs (ac, "PR to be unlocked"))
     return;
 
-  path = lookup_pr_path (av[0]); /* XXX what if it's null? */
+  path = lookup_pr_path (av[0]);
+  if (! path)
+    return;
+  
   if (unlock_pr (path))
-    printf ("%d PR %s unlocked.\r\n", CODE_OK, av[0]);
+    printf ("%d PR %s unlocked.\r\n", CODE_OK, get_cat_prid_from_path (path));
   xfree (path);
 }
 
 int
-check_locked (path, p)
-     char *path, *p;
+check_locked (path)
+     char *path;
 {
   struct stat buf;
-  char *lock_path = (char *) xmalloc (strlen (path) + 6);
+  char *lock_path = get_lock_path (path);
 
-  strcpy (lock_path, path);
-  strcat (lock_path, ".lock");
   if (stat (lock_path, &buf) != 0)
     {
       if (errno == ENOENT)
 	printf ("%d You must lock PR %s and get its current content first.\r\n",
-		CODE_PR_NOT_LOCKED, p);
+		CODE_PR_NOT_LOCKED, get_cat_prid_from_path (path));
       else
 	printf ("%d Could not detect if the lock for %s was there.\r\n",
-		CODE_FILE_ERROR, p);
+		CODE_FILE_ERROR, get_cat_prid_from_path (path));
+      xfree (lock_path);
       return 0;
     }
   xfree (lock_path);
@@ -510,9 +421,8 @@ GNATS_lkdb (ac, av)
     }
 
   /* This does the error for us.  */
-  lock_gnats ();
-
-  printf ("%d GNATS database is now locked.\r\n", CODE_OK);
+  if (lock_gnats () == 0)
+    printf ("%d GNATS database is now locked.\r\n", CODE_OK);
 }
 
 void
@@ -526,14 +436,249 @@ GNATS_undb (ac, av)
       return;
     }
 
-  if (unlock_gnats () < 0)
+  unlock_gnats ();
+}
+
+void
+GNATS_user (ac, av)
+     int ac;
+     char **av;
+{
+  char *path;
+  int   access;
+
+  if (ac == 0)
     {
-      printf ("%d failed unlocking GNATS database.\r\n",
+      printf ("%d User access level set to %s\r\n", CODE_OK, access_level_str (user_access));
+      return;
+    }
+  
+  if (ac != 2)
+    {
+      printf ("%d USER requires two arguments, username and password.\r\n",
 	      CODE_ERROR);
       return;
     }
+  
+  path = (char *) xmalloc (PATH_MAX);
+  sprintf (path, "%s/gnats-adm/%s", gnats_root, DB_ACCESS_FILE);
+  access = get_user_access (gnats_root, path, av[0], av[1]);
+  xfree (path);
 
-  printf ("%d GNATS database is now unlocked.\r\n", CODE_OK);
+  if (access < 0)
+    access = get_user_access (gnats_root, GLOBAL_ACCESS_FILE, av[0], av[1]);
+
+  if (access < 0)
+    access = ACCESS_NONE;
+  
+  if (access > host_access)
+    user_access = access;
+
+  if (user_access <= ACCESS_NONE)
+    {
+      syslog (LOG_ERR, "user not allowed access: %s/%s", av[0], av[1]);
+      printf ("%d You are not on the user access list: %s/%s.\r\n",
+              CODE_NO_ACCESS, av[0], av[1]);
+      exit (1);
+    }
+  
+  set_confidential_access ();
+
+  printf ("%d User access level set to %s\r\n", CODE_OK, access_level_str (user_access));
+}
+
+/* change database command */
+void
+GNATS_chdb (ac, av)
+     int ac;
+     char **av;
+{
+  static char *new_root = NULL;
+
+  char  *new_db = NULL;
+  char  *alias_db = NULL;
+  int    access;
+
+  struct stat buf;
+
+  if (! chk_nargs(ac, "Database name"))
+    return;
+
+  /* verify new root exists and is a directory */
+  new_db = av[0];
+
+  /* We only allow database aliases in gnatsd, not full pathnames */
+  alias_db = new_db;
+  new_db = lookup_conf_alias(alias_db);
+  if (!new_db)
+    {
+      printf ("%d Could not resolve database alias (%s).\r\n",
+	      CODE_FILE_ERROR, alias_db);
+      return;
+    }
+
+  if (stat(new_db, &buf) != 0)
+    {
+      if (errno == ENOENT)
+	printf ("%d Could not change to database (%s): does not exist.\r\n",
+		CODE_FILE_ERROR, new_db);
+      else
+	printf ("%d Error while getting status of (%s): errno = %d.\r\n",
+		CODE_FILE_ERROR, new_db, errno);
+      return;
+    }
+  else if ((buf.st_mode & S_IFMT) != S_IFDIR)
+    {
+      printf ("%d Specified database (%s) is not a directory.\r\n",
+	      CODE_FILE_ERROR, new_db);
+      return;
+    }
+
+  /* free existing copy then make new copy */
+  if (new_root)
+    xfree(new_root);
+  new_root = (char *) xstrdup(new_db);
+
+  /* re-initialize using new gnats_root */
+  gnats_root = new_root;
+
+  /* This doret stuff is wierd, but the following is necessary. <rickm@vsl.com> */
+  doret = 0;
+  init_gnats ("gnatsd");
+  doret = 1;
+
+  /* make sure that a host that is allowed to access one database does
+     not switch to a database where permission is not granted */
+  if (!allowed (current_host, current_addr, gnats_root, &access))
+    {
+      syslog (LOG_ERR, "host %s (%s) not allowed access for %s",
+              current_host, current_addr, gnats_root);
+      printf ("%d You are not on the host access list: %s (%s).\r\n",
+              CODE_NO_ACCESS, current_host, current_addr);
+      exit (1);
+    }
+
+  /* Clear the gnatsd --require-db flag and set user access to host default access for new db */
+  require_db  = 0;
+  user_access = access;
+  
+  set_confidential_access ();
+  
+  free_index(index_chain);
+  xfree (index_filename);
+  index_filename = (char *)NULL;
+  index_chain = get_index ();
+/*   if (index_chain == NULL) */
+/*     { */
+/*       printf ("%d GNATS server cannot read the index.\r\n", CODE_FILE_ERROR); */
+/*       exit (1); */
+/*     } */
+
+  printf ("%d Now accessing GNATS database '%s'\r\n", CODE_OK, gnats_root);
+}
+
+/* list databases command */
+void
+GNATS_dbls (ac, av)
+     int ac;
+     char **av;
+{
+  char **databases = NULL;
+  int  count = 0;
+  int  access;
+  int i;
+
+  /* get complete list */
+  get_conf_databases(&databases, &count);
+  
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  for(i = 0; i < count; i++)
+    {
+      /* only list hosts with permitted access */
+      if (allowed (current_host, current_addr, databases[i], &access))
+	{
+	  printf ("%s\r\n", databases[i]);
+	}
+      xfree(databases[i]);
+    }
+  printf (".\r\n");
+  xfree((char*)databases);
+}
+
+/* list database alaises command */
+void
+GNATS_dbla (ac, av)
+     int ac;
+     char **av;
+{
+  char **aliases = NULL;
+  int  count = 0;
+  int  access;
+  int  i;
+
+  get_conf_aliases(&aliases, &count);
+
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  
+  for(i = 0; i < count; i++)
+    {
+      /* only list hosts with permitted access */
+      if (allowed (current_host, current_addr, lookup_conf_alias(aliases[i]), &access))
+	{
+	  /* if alias is a dot send a double dot to avoid end of data
+             confusion */
+	  if (strcmp(aliases[i], ".") == 0)
+	    printf(".");
+	  printf ("%s\r\n", aliases[i]);
+	  xfree(aliases[i]);
+	}
+    }
+  printf (".\r\n");
+  xfree((char*) aliases);
+}
+
+void
+GNATS_chek (ac, av)
+     int ac;
+     char **av;
+{
+  char *given;
+  FILE *fp;
+
+  if (ac)
+    {
+      noargs ("CHEK");
+      return;
+    }
+
+  printf ("%d Ok.\r\n", CODE_OK); 
+  fflush (stdout);
+  given = get_text ();
+
+  if (! given)
+    {
+      return; /* XXX */
+    }
+
+  fp = fopen (given, "r");
+  if (fp == (FILE *)NULL)
+    {
+      printf ("%d Failed reading saved text.\r\n", CODE_ERROR);
+      goto checkout;
+    }
+
+  if (! check_pr (fp))
+    printf ("%d Errors found checking PR text.\r\n", CODE_ERROR);
+  else
+    printf ("%d PR text checked OK.\r\n", CODE_OK);
+  fclose (fp);
+
+  if (unlink (given) && errno != ENOENT)
+    syslog (LOG_ERR, "can't remove %s: %m", given);
+
+checkout:
+  xfree (given);
+  return;
 }
 
 void
@@ -551,7 +696,7 @@ GNATS_edit (ac, av)
   if (! path)
     return;
 
-  if (! check_locked (path, av[0]))
+  if (! check_locked (path))
     return;
 
   if (is_gnats_locked ())
@@ -567,7 +712,6 @@ GNATS_edit (ac, av)
 
   if (! given)
     {
-      unlock_pr (path);
       return; /* XXX */
     }
 
@@ -585,18 +729,18 @@ GNATS_edit (ac, av)
   if (! modify_pr (fp))
     printf ("%d Failed writing out PR.\r\n", CODE_ERROR);
   else
-    printf ("%d PR %s filed.\r\n", CODE_OK, av[0]);
+    printf ("%d PR %s filed.\r\n", CODE_OK, get_cat_prid_from_path (av[0]));
   fclose (fp);
+
   unlock_gnats ();
-  unlock_pr (path); /* XXX */
   /* unblock_signals (); */
   if (unlink (given) && errno != ENOENT)
     syslog (LOG_ERR, "can't remove %s: %m", given);
- editout:
+
+editout:
   xfree (path);
   xfree (given);
   return;
-
 }
 
 void
@@ -613,6 +757,7 @@ GNATS_mlpr (ac, av)
   if (! path)
     return;
 
+  /* GNATS_mlpr must call get_pr() because of the need to get the X_GNATS_NOTIFY header */
   if (get_pr (path, av[0], 1))
     {
       Category category;
@@ -641,20 +786,24 @@ GNATS_mlpr (ac, av)
 	printf ("%d- No responsible entry for %s.\r\n",
 		CODE_INVALID_RESPONSIBLE, category.person);
 
-      addrs[naddrs++] = category.person;
+      addrs[naddrs++] = strdup (category.person);
       if (category.notify[0])
-	addrs[naddrs++] = category.notify;
+        addrs[naddrs++] = strdup (category.notify);
       if (submitter.contact[0])
-	addrs[naddrs++] = submitter.contact;
+        addrs[naddrs++] = strdup (submitter.contact);
       if (submitter.notify[0])
-	addrs[naddrs++] = submitter.notify;
+        addrs[naddrs++] = strdup (submitter.notify);
       if (responsible)
-	addrs[naddrs++] = responsible->alias;
+        addrs[naddrs++] = strdup (responsible->alias);
       h = raw_header_value (X_GNATS_NOTIFY);
       if (h)
 	{
-	  char *h2 = strchr (h, '\n'); h2[0] = '\0';
-	  addrs[naddrs++] = h;
+	  char *h2 = strchr (h, '\n');
+	  if (h2 != NULL)
+	    {
+              h2[0] = '\0';
+	    }
+	  addrs[naddrs++] = strdup (h);
 	}
       addrs[naddrs] = NULL;
 
@@ -663,6 +812,8 @@ GNATS_mlpr (ac, av)
 	  free_responsible (responsible);
 	  xfree ((char *) responsible);
 	}
+      free_category  (&category);
+      free_submitter (&submitter);
 
       i = unify_addrs (addrs, naddrs);
       if (i == naddrs)
@@ -683,7 +834,7 @@ GNATS_mlpr (ac, av)
       printf ("%d End of list.\r\n", CODE_INFORMATION);
     }
   else
-    printf ("%d Invalid PR %s.\n", CODE_INVALID_PR, av[0]);
+    printf ("%d Invalid PR %s.\n", CODE_INVALID_PR, get_cat_prid_from_path (av[0]));
 
   xfree (path);
 }
@@ -707,7 +858,8 @@ GNATS_mlct (ac, av)
   printf ("%d-Addresses to notify for category %s:\r\n",
 	  CODE_INFORMATION, av[0]);
   printf ("%d-%s\r\n", CODE_INFORMATION, category.person);
-  printf ("%d-%s\r\n", CODE_INFORMATION, category.notify);
+  if (category.notify[0])
+    printf ("%d-%s\r\n", CODE_INFORMATION, category.notify);
   printf ("%d End of list.\r\n", CODE_INFORMATION);
 }
 
@@ -736,6 +888,34 @@ GNATS_mlsu (ac, av)
   printf ("%d End of list.\r\n", CODE_INFORMATION);
 }
 
+void
+GNATS_mlrs (ac, av)
+     int ac;
+     char **av;
+{
+  char *address;
+
+  if (ac < 3)
+    {
+      printf ("%d Must supply FULL flag, STRICT flag, and person's name.\r\n",
+	      CODE_ERROR);
+      return;
+    }
+
+  address = get_responsible_addr (atoi (av[0]), atoi (av[1]) , av[2]);
+
+  if (!address)
+    {
+      printf ("%d could not find the requested address.\r\n",
+	      CODE_ERROR);
+      return;
+    }
+  printf ("%d-Email address of %s:\r\n",
+	  CODE_INFORMATION, av[2]);
+  printf ("%d-%s\r\n", CODE_INFORMATION, address);
+  printf ("%d End of address.\r\n", CODE_INFORMATION);
+}
+
 void
 GNATS_type (ac, av)
      int ac;
@@ -816,14 +996,11 @@ GNATS_synp (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs (ac, "synopsis"))
     return;
 
-  if (buf)
-    xfree (buf);
-  synopsis = buf = strdup(*av);
+  searching = 1;
+  search->synopsis = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -832,14 +1009,11 @@ GNATS_rlse (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs (ac, "release"))
     return;
 
-  if (buf)
-    xfree (buf);
-  release = buf = strdup(*av);
+  searching = 1;
+  search->release = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -851,8 +1025,12 @@ GNATS_conf (ac, av)
   if (! chk_nargs (ac, "yes or no"))
     return;
 
-  searching = 1;
-  search->confidential = (char *) strdup (av[0]);
+  set_confidential_access ();
+  if (!search->confidential)
+    {
+      searching = 1;
+      search->confidential = (char *) strdup (av[0]);
+    }
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -887,15 +1065,11 @@ GNATS_orig (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs (ac, "name"))
     return;
 
-  if (buf)
-    xfree (buf);
-  originator = buf = strdup(*av);
-
+  searching = 1;
+  search->originator = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -904,15 +1078,11 @@ GNATS_clss (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs(ac, "class list"))
     return;
 
-  if (buf)
-    xfree (buf);
-  class = buf = strdup(*av);
-
+  searching = 1;
+  search->class = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -922,15 +1092,11 @@ GNATS_qrtr (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs(ac, "quarter"))
     return;
 
-  if (buf)
-    xfree (buf);
-  quarter = buf = strdup(*av);
-
+  searching = 1;
+  search->quarter = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -939,15 +1105,11 @@ GNATS_kywd (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs(ac, "keyword list"))
     return;
 
-  if (buf)
-    xfree (buf);
-  keywords = buf = strdup(*av);
-
+  searching = 1;
+  search->keywords = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -962,7 +1124,7 @@ GNATS_bfor (ac, av)
     return;
 
   t = get_date (*av, NULL);
-  if (t == -1)
+  if (t < 0)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
@@ -983,7 +1145,7 @@ GNATS_aftr (ac, av)
     return;
 
   t = get_date (*av, NULL);
-  if (t == -1)
+  if (t < 0)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
@@ -1018,10 +1180,11 @@ GNATS_abfr (ac, av)
     return;
 
   t = get_date (*av, NULL);
-  if (t == -1)
+  if (t < 0)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       arrived_before = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1034,6 +1197,27 @@ GNATS_araf (ac, av)
 {
   time_t t;
 
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t < 0)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      searching = 1;
+      arrived_after = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_cbfr (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
   if (! chk_nargs(ac, "date"))
     return;
 
@@ -1042,7 +1226,71 @@ GNATS_araf (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
-      arrived_after = t;
+      searching = 1;
+      closed_before = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_caft (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      searching = 1;
+      closed_after = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_mbfr (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      searching = 1;
+      modified_before = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_maft (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      searching = 1;
+      modified_after = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
 }
@@ -1058,7 +1306,7 @@ GNATS_vdat (ac, av)
     return;
 
   t = get_date (*av, NULL);
-  if (t == -1)
+  if (t < 0)
     printf ("%d Couldn't parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     printf ("%d Ok.\r\n", CODE_OK);
@@ -1112,7 +1360,6 @@ GNATS_mtxt (ac, av)
 {
   if (! chk_nargs (ac, "regular expression"))
     return;
-
   searching = 1;
   m_text_search = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
@@ -1139,6 +1386,10 @@ GNATS_rset (ac, av)
   xfree (search->confidential);
   xfree (search->severity);
   xfree (search->priority);
+  xfree (search->originator);
+  xfree (search->class);
+  xfree (search->release);
+  xfree (search->synopsis);
   memset (search, 0, sizeof (Index));
 
   clean_pr ();
@@ -1146,10 +1397,6 @@ GNATS_rset (ac, av)
   if (check_index ())
     new_index = 1;
 
-  xfree (originator);
-  originator = (char *) NULL;
-  xfree (class);
-  class = (char *) NULL;
   searching = skip_closed = 0;
   xfree (text_search);
   text_search = (char *) NULL;
@@ -1157,11 +1404,14 @@ GNATS_rset (ac, av)
   m_text_search = (char *) NULL;
 #ifdef GNATS_RELEASE_BASED
   required_before = required_after = (time_t)0;
-  xfree (quarter);
-  quarter = (char *) NULL;
-  xfree (keywords);
-  keywords = (char *) NULL;
+  xfree (search->quarter);
+  xfree (search->keywords);
 #endif
+  arrived_before  = arrived_after  = (time_t)0;
+  modified_before = modified_after = (time_t)0;
+  closed_before   = closed_after   = (time_t)0;
+
+  set_confidential_access ();
 
   if (new_index)
     printf ("%d Reset state...reloaded the index.\r\n", CODE_OK);
@@ -1356,7 +1606,7 @@ GNATS_auth (ac, av)
 			   peer.sin_addr.s_addr, &k.auth, keyfile);
       if (status != KSUCCESS)
 	{
-	  syslog (LOG_NOTICE, "auth failed for %s: %s",
+	  syslog (LOG_ERR, "auth failed for %s: %s",
 		  current_host, krb_get_err_text (status));
 	  printf ("%d Authentication failed: %s.\r\n",
 		  CODE_ERROR, krb_get_err_text (status));
@@ -1412,6 +1662,7 @@ GNATS_auth (ac, av)
 
       printf ("%d Okie-dokey.\r\n", CODE_OK);
       client_authorized = 1;
+      host_access = user_access = ACCESS_EDIT;
       return;
     }
 #endif
@@ -1468,6 +1719,51 @@ GNATS_lsub (ac, av)
     printf (".\r\n");
 }
 
+void
+GNATS_lsta (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LSTA");
+      return;
+    }
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  if (get_gnats_file (LIST_STATES, NULL) == 0)
+    printf (".\r\n");
+}
+
+void
+GNATS_lcla (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LCLA");
+      return;
+    }
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  if (get_gnats_file (LIST_CLASSES, NULL) == 0)
+    printf (".\r\n");
+}
+
+void
+GNATS_lcfg (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LCFG");
+      return;
+    }
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  if (get_gnats_file (LIST_CONFIG, NULL) == 0)
+    printf (".\r\n");
+}
+
 void
 GNATS_help (ac, av)
      int ac;
@@ -1485,6 +1781,12 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   LSUB                    list the available submitters\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   LSTA                    list the available states\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   LCLA                    list the available classes\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   LCFG                    list the GNATS config file\r\n",
+	  CODE_INFORMATION);
   printf ("%d-   CATG <category>         search for PRs in <category>\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   SYNP <synopsis>         search for PRs have the synopsis <synopsis>\r\n",
@@ -1515,9 +1817,17 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   ARAF <date>             find all PRs that arrived before <date>\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-   VDATE <date>            validate the format of <date>\r\n",
+  printf ("%d-   MBFR <date>             find all PRs that were modified before <date>\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-   SVTY <sevty>            search for PRs which are of the severity <sevty>\r\n",
+  printf ("%d-   MAFT <date>             find all PRs that were modified before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   CBFR <date>             find all PRs that were closed before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   CAFT <date>             find all PRs that were closed before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   VDAT <date>             validate the format of <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   SVTY <severity>         search for PRs which are of the severity <severity>\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   STAT <state>            search for PRs in the state <state>\r\n",
 	  CODE_INFORMATION);
@@ -1533,6 +1843,8 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   MLSU <submitter>        list who should get mail about <submitter> \r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   MLRS <full> <strict> <person>  give the email address for <person> \r\n",
+	  CODE_INFORMATION);
   printf ("%d-   TYPE <submitter>        give the type of a given submitter\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   RSET                    reset internal settings to initial startup\r\n",
@@ -1541,10 +1853,27 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   UNDB                    unlock the main GNATS database\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   LOCK <PR> <user> <pid>  lock <PR> for <user> and optional <pid> and return PR text\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   UNLK <PR>               unlock <PR>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   CHEK                    check PR text for errors\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   EDIT <PR>               check in edited <PR>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   CHDB <database>         change GNATS ROOT to <database>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   DBLS                    list databases known to server\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   DBLA                    list database aliases known to server\r\n",
+	  CODE_INFORMATION);
+
 #ifdef HAVE_KERBEROS
   printf ("%d-   AUTH <kerberos>	 Kerberos authentication\r\n",
 	  CODE_INFORMATION);
 #endif
+  printf ("%d-   USER <userid> <passwd>	 UserId and password for database access.\r\n",
+	  CODE_INFORMATION);
   printf ("%d-The search itself is taken by either QURY, SUMM, or FULL.  Each\r\n",
 	  CODE_INFORMATION);
   printf ("%d-  may be invoked with no arguments, which will search the entire\r\n",
@@ -1559,9 +1888,9 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-  FULL command will show the entire header and body of the PR.\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-You may also use the option SQLF, which will give you an SQL format\r\n",
+  printf ("%d-You may also use the option SQLF or SQL2, which will give you an SQL\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-  line for each PR, which you can bring into your own database.\r\n",
+  printf ("%d-  format line for each PR, which you can bring into your own database.\r\n",
 	  CODE_INFORMATION);
   printf ("%d End of HELP info for GNATS daemon.\r\n", CODE_INFORMATION);
 }
