@@ -396,7 +396,7 @@ char *
 check_if_reply ()
 {
   char token[STR_MAX];
-  char *path;
+  char *path = NULL;
   struct stat buf;
   Category cat;
   char *s, *c, *cat_name;
@@ -788,7 +788,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
       if (i == SUBJECT)
 	continue;
       if (*header_value(i))
-	fprintf(msg, "%s%s", header_name(i), header_value(i));
+	fprintf(msg, "%s %s", header_name(i), header_value(i));
     }
 #endif /* DONT_USE_RESENT */
 
