@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from getdate.y
+/*  A Bison parser, made from ./getdate.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -18,7 +18,7 @@
 #define	tZONE	268
 #define	tDST	269
 
-#line 1 "getdate.y"
+#line 1 "./getdate.y"
 
 /*
 **  Originally written by Steven M. Bellovin <smb@research.att.com> while
@@ -170,7 +170,7 @@ static time_t	yyRelMonth;
 static time_t	yyRelSeconds;
 
 
-#line 153 "getdate.y"
+#line 153 "./getdate.y"
 typedef union {
     time_t		Number;
     enum _MERIDIAN	Meridian;
@@ -330,7 +330,7 @@ static const short yycheck[] = {     0,
     13,    47
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/bugshack/segv/jsm/local/share/bison.simple"
+#line 3 "/usr/share/misc/bison.simple"
 /* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
@@ -544,7 +544,7 @@ __yy_memcpy (char *to, char *from, unsigned int count)
 #endif
 #endif
 
-#line 217 "/bugshack/segv/jsm/local/share/bison.simple"
+#line 217 "/usr/share/misc/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -873,37 +873,37 @@ yyreduce:
   switch (yyn) {
 
 case 3:
-#line 171 "getdate.y"
+#line 171 "./getdate.y"
 {
 	    yyHaveTime++;
 	;
     break;}
 case 4:
-#line 174 "getdate.y"
+#line 174 "./getdate.y"
 {
 	    yyHaveZone++;
 	;
     break;}
 case 5:
-#line 177 "getdate.y"
+#line 177 "./getdate.y"
 {
 	    yyHaveDate++;
 	;
     break;}
 case 6:
-#line 180 "getdate.y"
+#line 180 "./getdate.y"
 {
 	    yyHaveDay++;
 	;
     break;}
 case 7:
-#line 183 "getdate.y"
+#line 183 "./getdate.y"
 {
 	    yyHaveRel++;
 	;
     break;}
 case 9:
-#line 189 "getdate.y"
+#line 189 "./getdate.y"
 {
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
@@ -912,7 +912,7 @@ case 9:
 	;
     break;}
 case 10:
-#line 195 "getdate.y"
+#line 195 "./getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -921,7 +921,7 @@ case 10:
 	;
     break;}
 case 11:
-#line 201 "getdate.y"
+#line 201 "./getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -931,7 +931,7 @@ case 11:
 	;
     break;}
 case 12:
-#line 208 "getdate.y"
+#line 208 "./getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -940,7 +940,7 @@ case 12:
 	;
     break;}
 case 13:
-#line 214 "getdate.y"
+#line 214 "./getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -951,56 +951,56 @@ case 13:
 	;
     break;}
 case 14:
-#line 224 "getdate.y"
+#line 224 "./getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSToff;
 	;
     break;}
 case 15:
-#line 228 "getdate.y"
+#line 228 "./getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSTon;
 	;
     break;}
 case 16:
-#line 233 "getdate.y"
+#line 233 "./getdate.y"
 {
 	    yyTimezone = yyvsp[-1].Number;
 	    yyDSTmode = DSTon;
 	;
     break;}
 case 17:
-#line 239 "getdate.y"
+#line 239 "./getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 18:
-#line 243 "getdate.y"
+#line 243 "./getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	;
     break;}
 case 19:
-#line 247 "getdate.y"
+#line 247 "./getdate.y"
 {
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 20:
-#line 253 "getdate.y"
+#line 253 "./getdate.y"
 {
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 21:
-#line 257 "getdate.y"
+#line 257 "./getdate.y"
 {
 	    yyMonth = yyvsp[-4].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1008,7 +1008,7 @@ case 21:
 	;
     break;}
 case 22:
-#line 262 "getdate.y"
+#line 262 "./getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
@@ -1017,7 +1017,7 @@ case 22:
 	;
     break;}
 case 23:
-#line 268 "getdate.y"
+#line 268 "./getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
@@ -1026,14 +1026,14 @@ case 23:
 	;
     break;}
 case 24:
-#line 274 "getdate.y"
+#line 274 "./getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 25:
-#line 278 "getdate.y"
+#line 278 "./getdate.y"
 {
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1041,14 +1041,14 @@ case 25:
 	;
     break;}
 case 26:
-#line 283 "getdate.y"
+#line 283 "./getdate.y"
 {
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	;
     break;}
 case 27:
-#line 287 "getdate.y"
+#line 287 "./getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1056,68 +1056,68 @@ case 27:
 	;
     break;}
 case 28:
-#line 294 "getdate.y"
+#line 294 "./getdate.y"
 {
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMonth = -yyRelMonth;
 	;
     break;}
 case 30:
-#line 301 "getdate.y"
+#line 301 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	;
     break;}
 case 31:
-#line 304 "getdate.y"
+#line 304 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	;
     break;}
 case 32:
-#line 307 "getdate.y"
+#line 307 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[0].Number * 60L;
 	;
     break;}
 case 33:
-#line 310 "getdate.y"
+#line 310 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	;
     break;}
 case 34:
-#line 313 "getdate.y"
+#line 313 "./getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	;
     break;}
 case 35:
-#line 316 "getdate.y"
+#line 316 "./getdate.y"
 {
 	    yyRelSeconds++;
 	;
     break;}
 case 36:
-#line 319 "getdate.y"
+#line 319 "./getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 37:
-#line 322 "getdate.y"
+#line 322 "./getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 38:
-#line 325 "getdate.y"
+#line 325 "./getdate.y"
 {
 	    yyRelMonth += yyvsp[0].Number;
 	;
     break;}
 case 39:
-#line 330 "getdate.y"
+#line 330 "./getdate.y"
 {
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 		yyYear = yyvsp[0].Number;
@@ -1145,20 +1145,20 @@ case 39:
 	;
     break;}
 case 40:
-#line 357 "getdate.y"
+#line 357 "./getdate.y"
 {
 	    yyval.Meridian = MER24;
 	;
     break;}
 case 41:
-#line 360 "getdate.y"
+#line 360 "./getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	;
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 543 "/bugshack/segv/jsm/local/share/bison.simple"
+#line 543 "/usr/share/misc/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1378,7 +1378,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 365 "getdate.y"
+#line 365 "./getdate.y"
 
 
 /* Month and day table. */
@@ -1457,7 +1457,9 @@ static TABLE const TimezoneTable[] = {
     { "ut",	tZONE,     HOUR ( 0) },	/* Universal (Coordinated) */
     { "utc",	tZONE,     HOUR ( 0) },
     { "wet",	tZONE,     HOUR ( 0) },	/* Western European */
+    { "west",   tZONE,     HOUR ( 0) }, /* Alias for Western European */
     { "bst",	tDAYZONE,  HOUR ( 0) },	/* British Summer */
+    { "gb",     tDAYZONE,  HOUR ( 0) }, /* GMT with daylight savings */
     { "wat",	tZONE,     HOUR ( 1) },	/* West Africa */
     { "at",	tZONE,     HOUR ( 2) },	/* Azores */
 #if	0
@@ -1472,8 +1474,8 @@ static TABLE const TimezoneTable[] = {
     { "ndt",	tDAYZONE,  HOUR (3.5) },	/* Newfoundland Daylight */
 #endif
     { "ast",	tZONE,     HOUR ( 4) },	/* Atlantic Standard */
+    { "sat",    tZONE,     HOUR ( 4) }, /* Chile */
     { "adt",	tDAYZONE,  HOUR ( 4) },	/* Atlantic Daylight */
-    { "eest",   tZONE,     -HOUR (2) }, /* Eastern European Standard Time */
     { "est",	tZONE,     HOUR ( 5) },	/* Eastern Standard */
     { "edt",	tDAYZONE,  HOUR ( 5) },	/* Eastern Daylight */
     { "cst",	tZONE,     HOUR ( 6) },	/* Central Standard */
@@ -1482,7 +1484,9 @@ static TABLE const TimezoneTable[] = {
     { "mdt",	tDAYZONE,  HOUR ( 7) },	/* Mountain Daylight */
     { "pst",	tZONE,     HOUR ( 8) },	/* Pacific Standard */
     { "pdt",	tDAYZONE,  HOUR ( 8) },	/* Pacific Daylight */
+    { "akst",   tZONE,     HOUR ( 9) }, /* Alaska Standard */
     { "yst",	tZONE,     HOUR ( 9) },	/* Yukon Standard */
+    { "akdt",   tDAYZONE,  HOUR ( 9) }, /* Alaska Daylight */
     { "ydt",	tDAYZONE,  HOUR ( 9) },	/* Yukon Daylight */
     { "hst",	tZONE,     HOUR (10) },	/* Hawaii Standard */
     { "hdt",	tDAYZONE,  HOUR (10) },	/* Hawaii Daylight */
@@ -1494,15 +1498,21 @@ static TABLE const TimezoneTable[] = {
     { "ces",    tDAYZONE,  -HOUR (1) }, /* Central European Summer */
     { "cest",   tDAYZONE,  -HOUR (1) }, /* Central European Summer */
     { "met",	tZONE,     -HOUR (1) },	/* Middle European */
+    { "mez",    tZONE,     -HOUR (1) }, /* Middle European */
     { "mewt",	tZONE,     -HOUR (1) },	/* Middle European Winter */
     { "mest",	tDAYZONE,  -HOUR (1) },	/* Middle European Summer */
+    { "metdst", tDAYZONE,  -HOUR (1) }, /* An alias for mest used by HP-UX */
     { "mesz",	tDAYZONE,  -HOUR (1) },	/* Middle European Summer */
     { "swt",	tZONE,     -HOUR (1) },	/* Swedish Winter */
     { "sst",	tDAYZONE,  -HOUR (1) },	/* Swedish Summer */
     { "fwt",	tZONE,     -HOUR (1) },	/* French Winter */
     { "fst",	tDAYZONE,  -HOUR (1) },	/* French Summer */
     { "eet",	tZONE,     -HOUR (2) },	/* Eastern Europe, USSR Zone 1 */
-    { "bt",	tZONE,     -HOUR (3) },	/* Baghdad, USSR Zone 2 */
+    { "eest",   tZONE,     -HOUR (2) }, /* Eastern European Standard Time */
+    { "sast",   tZONE,     -HOUR (2) }, /* South African Standard */
+    { "msd",    tDAYZONE,  -HOUR (2) }, /* Moscow Daylight (msd=msk) */
+    { "msk",    tZONE,     -HOUR (3) }, /* Moscow */
+    { "bt",     tZONE,     -HOUR (3) }, /* Baghdad, USSR Zone 2 */
 #if 0
     { "it",	tZONE,     -HOUR (3.5) },/* Iran */
 #endif
@@ -1523,8 +1533,12 @@ static TABLE const TimezoneTable[] = {
 #if 0
     { "jt",	tZONE,     -HOUR (7.5) },/* Java (3pm in Cronusland!) */
 #endif
+    { "awst",   tZONE,     -HOUR (8) }, /* West Australian Standard */
     { "cct",	tZONE,     -HOUR (8) },	/* China Coast, USSR Zone 7 */
+    { "pht",    tZONE,     -HOUR (8) }, /* Asia Manila */
+    { "wst",    tZONE,     -HOUR (8) }, /* West Australian Standard */
     { "jst",	tZONE,     -HOUR (9) },	/* Japan Standard, USSR Zone 8 */
+    { "rok",    tZONE,     -HOUR (9) }, /* Republic of Korea */
 #if 0
     { "cast",	tZONE,     -HOUR (9.5) },/* Central Australian Standard */
     { "cadt",	tDAYZONE,  -HOUR (9.5) },/* Central Australian Daylight */
