@@ -44,6 +44,12 @@ extern PR_entry pr[NUM_PR_ITEMS];
 /* Head of linked list of responsible people. */
 extern Responsible *responsible_chain;
 
+/* Head of linked list of classes. */
+extern Classes *class_chain;
+
+/* Head of linked list of states. */
+extern States *state_chain;
+
 /* Where to keep the static index if necessary.  */
 extern Index *index_chain;
 
@@ -99,9 +105,19 @@ extern void append_report		PARAMS((FILE *, char *));
 extern int find_category		PARAMS((Category*, char *));
 extern int find_submitter		PARAMS((Submitter*, char *));
 extern int init_responsibles		PARAMS((void));
+extern int init_classes  		PARAMS((void));
+extern int check_class_type		PARAMS((char*, char *));
+extern char *get_class_type		PARAMS((char*));
+extern char *get_class_enum_field	PARAMS((void));
+extern int init_states  		PARAMS((void));
+extern int check_state_type		PARAMS((char*, char *));
+extern char *get_state_type		PARAMS((char*));
+extern char *get_state_enum_field	PARAMS((void));
 extern void free_responsible		PARAMS((Responsible*));
 extern void free_category		PARAMS((Category *));
 extern void free_submitter		PARAMS((Submitter *));
+extern void free_state  		PARAMS((States *));
+extern char *get_responsible_addr       PARAMS((int, int, char *));
 
 /* in resp_lookup.c */
 extern Responsible* get_responsible		PARAMS((Category *));
