@@ -18,10 +18,6 @@ You should have received a copy of the GNU General Public License
 along with GNU GNATS; see the file COPYING.  If not, write to the Free
 Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-
 /* The information we need for a reply from the server.  */
 typedef struct _reply
 {
@@ -36,6 +32,9 @@ typedef enum _msg_type {
 
 /* in gnatsd.c */
 extern MsgType get_line			PARAMS((char *, int, int));
+extern int     allowed                  PARAMS((char *, char *, char *, int *));
+extern int     get_user_access          PARAMS((char *, char *, char *, char *));
+extern void    set_confidential_access  PARAMS((void));
 
 /* cmds.c */
 extern void GNATS_helo			PARAMS((int, char**));
@@ -43,15 +42,21 @@ extern void GNATS_quit			PARAMS((int, char**));
 extern void GNATS_qury			PARAMS((int, char**));
 extern void GNATS_summ			PARAMS((int, char**));
 extern void GNATS_sqlf			PARAMS((int, char**));
+extern void GNATS_sql2			PARAMS((int, char**));
 extern void GNATS_full			PARAMS((int, char**));
 extern void GNATS_lock			PARAMS((int, char**));
 extern void GNATS_unlk			PARAMS((int, char**));
 extern void GNATS_lkdb			PARAMS((int, char**));
 extern void GNATS_undb			PARAMS((int, char**));
+extern void GNATS_chdb			PARAMS((int, char**));
+extern void GNATS_dbls			PARAMS((int, char**));
+extern void GNATS_dbla			PARAMS((int, char**));
+extern void GNATS_chek			PARAMS((int, char**));
 extern void GNATS_edit			PARAMS((int, char**));
 extern void GNATS_mlpr			PARAMS((int, char**));
 extern void GNATS_mlct			PARAMS((int, char**));
 extern void GNATS_mlsu			PARAMS((int, char**));
+extern void GNATS_mlrs			PARAMS((int, char**));
 extern void GNATS_type			PARAMS((int, char**));
 extern void GNATS_resp			PARAMS((int, char**));
 extern void GNATS_catg			PARAMS((int, char**));
@@ -65,6 +70,10 @@ extern void GNATS_clss			PARAMS((int, char**));
 extern void GNATS_rlse			PARAMS((int, char**));
 extern void GNATS_abfr			PARAMS((int, char**));
 extern void GNATS_araf			PARAMS((int, char**));
+extern void GNATS_mbfr			PARAMS((int, char**));
+extern void GNATS_maft			PARAMS((int, char**));
+extern void GNATS_cbfr			PARAMS((int, char**));
+extern void GNATS_caft			PARAMS((int, char**));
 extern void GNATS_vdat			PARAMS((int, char**));
 extern void GNATS_orig			PARAMS((int, char**));
 extern void GNATS_prio			PARAMS((int, char**));
@@ -78,9 +87,13 @@ extern void GNATS_text			PARAMS((int, char**));
 extern void GNATS_mtxt			PARAMS((int, char**));
 extern void GNATS_rset			PARAMS((int, char**));
 extern void GNATS_auth			PARAMS((int, char**));
+extern void GNATS_user			PARAMS((int, char**));
 extern void GNATS_lcat			PARAMS((int, char**));
+extern void GNATS_lcfg			PARAMS((int, char**));
+extern void GNATS_lcla			PARAMS((int, char**));
 extern void GNATS_lres			PARAMS((int, char**));
 extern void GNATS_lsub			PARAMS((int, char**));
+extern void GNATS_lsta			PARAMS((int, char**));
 extern void GNATS_load			PARAMS((int, char**));
 extern void GNATS_chst			PARAMS((int, char**));
 extern void GNATS_chre			PARAMS((int, char**));
@@ -103,7 +116,9 @@ extern void GNATS_kywd			PARAMS((int, char**));
 #include <netinet/in.h> /* for sockaddr_in */
 #include <arpa/inet.h>	/* for inet_ntoa decl */
 #include <sys/param.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h>	/* for hostent and MAXHOSTNAMELEN */
+#endif
 #include <sys/socket.h>	/* for AF_INET */
 #include <sys/time.h>	/* for timeval */
 #ifdef HAVE_SYS_SELECT_H
@@ -114,6 +129,17 @@ extern void GNATS_kywd			PARAMS((int, char**));
 #define REPLY_CONT	1
 #define REPLY_END	2
 
+/* gnatsd flags, including host and user access levels */
+#define FLAG_MASK	0xF00
+#define FLAG_ONE_ARG	0x100
+#define ACCESS_MASK     0xFF
+#define ACCESS_DENY     0x01
+#define ACCESS_NONE     0x02
+#define ACCESS_VIEW     0x03
+#define ACCESS_VIEWCONF 0x04
+#define ACCESS_EDIT     0x05
+#define ACCESS_ADMIN    0x06
+
 /* Used if Kerberos version 4 authentication is enabled.  */
 #define GNATS_KRB4_VERSIONID		"GnatsK1.0"
 #define GNATS_KRB4_PRINCIPAL_NAME	gnats_user
