@@ -60,6 +60,24 @@ get_gnats_file (type, file)
 	strcat (outf, ".res");
       infile = RESPONSIBLE_FILE;
     }
+  else if (type == LIST_STATES)
+    {
+      if (outf)
+	strcat (outf, ".sta");
+      infile = STATES;
+    }
+  else if (type == LIST_CLASSES)
+    {
+      if (outf)
+	strcat (outf, ".cla");
+      infile = CLASSES;
+    }
+  else if (type == LIST_CONFIG)
+    {
+      if (outf)
+	strcat (outf, ".cfg");
+      infile = CONFIG;
+    }
   else
     {
       if (outf)
@@ -71,31 +89,65 @@ get_gnats_file (type, file)
     {
       fpout = fopen (outf, "w");
       if (fpout == (FILE *)NULL)
-	return -1; /* XXX */
+        return -1; /* XXX */
     }
 
-  sprintf (path, "%s/gnats-adm/%s", gnats_root, infile);
-  fpin = fopen (path, "r");
-  if (fpin == (FILE *)NULL)
-    return -1; /* XXX */
-
-  while (read_string (line, fpin) > 0)
-    if (line[0] != '#' && line[0] != ' ' && line[0] != '\n')
-      {
-	if (! outf && doret)
-	  {
-	    char *s = strchr (line, '\n');
-	    if (s)
+  if (type == LIST_CLASSES)
+    {
+      if (class_chain != NULL)
+        {
+          Classes *c;
+          for (c = class_chain; c; c = c->next)
+            {
+	      if (! outf && doret)
+                fprintf (fpout, "%s:%s:%s\r\n", c->key, c->type, c->description);
+              else
+                fprintf (fpout, "%s:%s:%s\n",   c->key, c->type, c->description);
+            }
+        }
+      else
+        return -1;
+    }
+  else if (type == LIST_STATES)
+    {
+      if (state_chain != NULL)
+        {
+          States *s;
+          for (s = state_chain; s; s = s->next)
+            {
+	      if (! outf && doret)
+                fprintf (fpout, "%s:%s:%s\r\n", s->key, s->type, s->description);
+              else
+                fprintf (fpout, "%s:%s:%s\n",   s->key, s->type, s->description);
+            }
+        }
+      else
+        return -1;
+    }
+  else
+    {
+      sprintf (path, "%s/gnats-adm/%s", gnats_root, infile);
+      fpin = fopen (path, "r");
+      if (fpin == (FILE *)NULL)
+          return -1; /* XXX */
+      while (read_string (line, fpin) > 0)
+        if (line[0] != '#' && line[0] != ' ' && line[0] != '\n')
+          {
+	    if (! outf && doret)
 	      {
-		s[0] = '\0';
-		fprintf (fpout, "%s\r\n", line);
+	        char *s = strchr (line, '\n');
+	        if (s)
+	          {
+                    s[0] = '\0';
+                    fprintf (fpout, "%s\r\n", line);
+	          }
 	      }
-	  }
-	else
-	  fputs (line, fpout);
-      }
+	    else
+              fputs (line, fpout);
+          }
+      fclose (fpin);
+    }    
 
-  fclose (fpin);
   if (outf)
     fclose (fpout);
 
