@@ -22,6 +22,12 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #include "gnats.h"
 #include "query.h"
 
+/* If 1, we're running the daemon.  */
+int is_daemon = 0;
+
+/* If 1, we're running a network client.  */
+int is_client = 1;
+
 /* The descriptor for talking to the server in non-stdio ways.  */
 extern int sockfd;
 
@@ -40,6 +46,18 @@ char *arrived_before_string = NULL;
 /* If set, match PRs with an Arrival-Date: field after this value.  */
 char *arrived_after_string = NULL;
 
+/* If set, match PRs with a Last-Modified: field before this value.  */
+char *modified_before_string = NULL;
+
+/* If set, match PRs with a Last-Modified: field after this value.  */
+char *modified_after_string = NULL;
+
+/* If set, match PRs with a Closed-Date: field before this value.  */
+char *closed_before_string = NULL;
+
+/* If set, match PRs with a Closed-Date: field after this value.  */
+char *closed_after_string = NULL;
+
 #ifdef GNATS_RELEASE_BASED
 /* If set, match PRs with a Date-Required: field before this value.  */
 char *required_before_string = NULL;
@@ -104,6 +122,8 @@ struct option long_options[] =
   {"help", 0, NULL, 'h'},
   {"host", 1, NULL, 'H'},
   {"port", 1, NULL, PORT_OPTION},
+  {"user", 1, NULL, 'v'},
+  {"passwd", 1, NULL, 'w'},
   {"multitext", 1, NULL, 'm'},
   {"originator", 1, NULL, 'O'},
   {"release", 1, NULL, 'A'},
@@ -120,6 +140,7 @@ struct option long_options[] =
   {"severity", 1, NULL, 'e'},
   {"skip-closed", 0, NULL, 'x'},
   {"sql", 0, NULL, 'i'},
+  {"sql2", 0, NULL, 'I'},
   {"state", 1, NULL, 's'},
   {"summary", 0, NULL, 'q'},
   {"submitter", 1, NULL, 'S'},
@@ -130,9 +151,16 @@ struct option long_options[] =
 #endif
   {"arrived-before", 1, NULL, 'b'},
   {"arrived-after", 1, NULL, 'a'},
+  {"modified-before", 1, NULL, 'B'},
+  {"modified-after", 1, NULL, 'M'},
+  {"closed-before", 1, NULL, 'z'},
+  {"closed-after", 1, NULL, 'Z'},
   {"list-categories", 0, NULL, 'j'},
+  {"list-classes", 0, NULL, 'J'},
   {"list-responsible", 0, NULL, 'k'},
   {"list-submitters", 0, NULL, 'l'},
+  {"list-states", 0, NULL, 'T'},
+  {"list-config", 0, NULL, 'G'},
   {"version", 0, NULL, 'V'},
   {NULL, 0, NULL, 0}
 };
@@ -170,6 +198,24 @@ ask_about (info, args)
 	    fprintf (stderr, "%s: writing `LSUB'\n", program_name);
 	  fprintf (serv_write, "LSUB\r\n");
 	}
+      else if (list_format == LIST_STATES)
+	{
+	  if (debug)
+	    fprintf (stderr, "%s: writing `LSTA'\n", program_name);
+	  fprintf (serv_write, "LSTA\r\n");
+	}
+      else if (list_format == LIST_CLASSES)
+	{
+	  if (debug)
+	    fprintf (stderr, "%s: writing `LCLA'\n", program_name);
+	  fprintf (serv_write, "LCLA\r\n");
+	}
+      else if (list_format == LIST_CONFIG)
+	{
+	  if (debug)
+	    fprintf (stderr, "%s: writing `LCFG'\n", program_name);
+	  fprintf (serv_write, "LCFG\r\n");
+	}
       return;
     }
 
@@ -245,53 +291,53 @@ ask_about (info, args)
       get_reply ();
     }
 
-  if (originator)
+  if (info->originator)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `ORIG %s'\n", program_name, originator);
-      fprintf (serv_write, "ORIG %s\r\n", originator);
+	fprintf (stderr, "%s: writing `ORIG %s'\n", program_name, info->originator);
+      fprintf (serv_write, "ORIG %s\r\n", info->originator);
       get_reply ();
     }
 
-  if (class)
+  if (info->class)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `CLSS %s'\n", program_name, class);
-      fprintf (serv_write, "CLSS %s\r\n", class);
+	fprintf (stderr, "%s: writing `CLSS %s'\n", program_name, info->class);
+      fprintf (serv_write, "CLSS %s\r\n", info->class);
       get_reply ();
     }
 
 #ifdef GNATS_RELEASE_BASED
-  if (quarter)
+  if (info->quarter)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `QRTR %s'\n", program_name, quarter);
-      fprintf (serv_write, "QRTR %s\r\n", quarter);
+	fprintf (stderr, "%s: writing `QRTR %s'\n", program_name, info->quarter);
+      fprintf (serv_write, "QRTR %s\r\n", info->quarter);
       get_reply ();
     }
 
-  if (keywords)
+  if (info->keywords)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `KYWD %s'\n", program_name, keywords);
-      fprintf (serv_write, "KYWD %s\r\n", keywords);
+	fprintf (stderr, "%s: writing `KYWD %s'\n", program_name, info->keywords);
+      fprintf (serv_write, "KYWD %s\r\n", info->keywords);
       get_reply ();
     }
 #endif
 
-  if (synopsis)
+  if (info->synopsis)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `SYNP %s'\n", program_name, synopsis);
-      fprintf (serv_write, "SYNP %s\r\n", synopsis);
+	fprintf (stderr, "%s: writing `SYNP %s'\n", program_name, info->synopsis);
+      fprintf (serv_write, "SYNP %s\r\n", info->synopsis);
       get_reply ();
     }
 
-  if (release)
+  if (info->release)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `RLSE %s'\n", program_name, release);
-      fprintf (serv_write, "RLSE %s\r\n", release);
+	fprintf (stderr, "%s: writing `RLSE %s'\n", program_name, info->release);
+      fprintf (serv_write, "RLSE %s\r\n", info->release);
       get_reply ();
     }
 
@@ -330,6 +376,42 @@ ask_about (info, args)
       get_reply ();
     }
 
+  if (modified_before_string)
+    {
+      if (debug)
+	fprintf (stderr, "%s: writing `MBFR %s'\n", program_name,
+		 modified_before_string);
+      fprintf (serv_write, "MBFR %s\r\n", modified_before_string);
+      get_reply ();
+    }
+
+  if (modified_after_string)
+    {
+      if (debug)
+	fprintf (stderr, "%s: writing `MAFT %s'\n", program_name,
+		 modified_after_string);
+      fprintf (serv_write, "MAFT %s\r\n", modified_after_string);
+      get_reply ();
+    }
+
+  if (closed_before_string)
+    {
+      if (debug)
+	fprintf (stderr, "%s: writing `CBFR %s'\n", program_name,
+		 closed_before_string);
+      fprintf (serv_write, "CBFR %s\r\n", closed_before_string);
+      get_reply ();
+    }
+
+  if (closed_after_string)
+    {
+      if (debug)
+	fprintf (stderr, "%s: writing `CAFT %s'\n", program_name,
+		 closed_after_string);
+      fprintf (serv_write, "CAFT %s\r\n", closed_after_string);
+      get_reply ();
+    }
+
 #ifdef GNATS_RELEASE_BASED
   if (required_before_string)
     {
@@ -356,6 +438,8 @@ ask_about (info, args)
     cmd = "FULL";
   else if (query_format & FORMAT_SQL)
     cmd = "SQLF";
+  else if (query_format & FORMAT_SQL2)
+    cmd = "SQL2";
   else
     cmd = "QURY";
 
@@ -390,7 +474,10 @@ main (argc, argv)
   int optc;
   Index *info;
   int formats = 0, lists = 0;
+  char *user = "anonymous", *passwd = "guest";
 
+  outfp = stdout;
+  
   program_name = basename (argv[0]);
   if (argc == 1)
     usage (1);
@@ -401,22 +488,16 @@ main (argc, argv)
   bzero ((Index *) info, sizeof (Index));
 
 #ifdef GNATS_RELEASE_BASED
-  while ((optc = getopt_long (argc, argv, "A:c:C:Dd:e:K:L:m:o:O:p:PQ:s:S:r:t:u:U:y:VFixhqH:Rjkl",
+  while ((optc = getopt_long (argc, argv, "a:A:b:B:c:C:Dd:e:K:L:m:M:o:O:p:PQ:s:S:r:t:u:U:v:w:y:z:Z:VFGiIxhqH:RjJklT",
 #else
-  while ((optc = getopt_long (argc, argv, "A:c:C:Dd:e:L:m:o:O:p:Ps:S:r:t:y:VFixhqH:Rjkl",
+  while ((optc = getopt_long (argc, argv, "a:A:b:B:c:C:Dd:e:L:m:M:o:O:p:Ps:S:r:t:v:w:y:z:Z:VFGiIxhqH:RjJklT",
 #endif
 			      long_options, (int *) 0)) != EOF)
     {
       switch (optc)
 	{
 	case 'd':
-#if 0
-	  spec_gnats_root = optarg;
-#else
-	  fprintf (stderr, "%s: directory option not valid for network query-pr\n",
-		   program_name);
-	  exit (1);
-#endif
+	  gnats_root = optarg;
 	  break;
 
 	case 'D':
@@ -424,7 +505,7 @@ main (argc, argv)
 	  break;
 
 	case 'A':
-	  release = optarg;
+	  info->release = optarg;
 	  break;
 
 	case 'a':
@@ -435,6 +516,22 @@ main (argc, argv)
 	  arrived_before_string = optarg;
 	  break;
 
+	case 'M':
+	  modified_after_string = optarg;
+	  break;
+
+	case 'B':
+	  modified_before_string = optarg;
+	  break;
+
+	case 'Z':
+	  closed_after_string = optarg;
+	  break;
+
+	case 'z':
+	  closed_before_string = optarg;
+	  break;
+
 	case 'o':
 	  if (strcmp (optarg, "-") && !restricted)
 	    {
@@ -473,16 +570,16 @@ main (argc, argv)
 	  break;
 
 	case 'L':
-	  class = optarg;
+	  info->class = optarg;
 	  break;
 
 #ifdef GNATS_RELEASE_BASED
 	case 'K':
-	  keywords = optarg;
+	  info->keywords = optarg;
 	  break;
 
 	case 'Q':
-	  quarter = optarg;
+	  info->quarter = optarg;
 	  break;
 #endif
 
@@ -491,7 +588,7 @@ main (argc, argv)
 	  break;
 
 	case 'O':
-	  originator = optarg;
+	  info->originator = optarg;
 	  break;
 
 	case 'p':
@@ -532,6 +629,11 @@ main (argc, argv)
 	  formats++;
 	  break;
 
+	case 'I':
+	  query_format |= FORMAT_SQL2;
+	  formats++;
+	  break;
+
 	case 'q':
 	  query_format |= FORMAT_SUMM;
 	  formats++;
@@ -556,6 +658,21 @@ main (argc, argv)
 	  lists++;
 	  break;
 
+	case 'T':
+	  list_format = LIST_STATES;
+	  lists++;
+	  break;
+
+	case 'J':
+	  list_format = LIST_CLASSES;
+	  lists++;
+	  break;
+
+	case 'G':
+	  list_format = LIST_CONFIG;
+	  lists++;
+	  break;
+
 #ifdef GNATS_RELEASE_BASED
 	case 'u':
 	  required_before_string = optarg;
@@ -567,7 +684,7 @@ main (argc, argv)
 #endif
 
 	case 'y':
-	  synopsis = optarg;
+	  info->synopsis = optarg;
 	  break;
 
 	case 'H':
@@ -578,6 +695,14 @@ main (argc, argv)
 	  port = atoi (optarg);
 	  break;
 
+        case 'v':
+          user = optarg;
+          break;
+
+        case 'w':
+          passwd = optarg;
+          break;
+
 	case 'h':
 	  usage (0);
 	  break;
@@ -596,12 +721,12 @@ main (argc, argv)
 
   if (lists > 1)
     {
-      fprintf (stderr, "%s: only one of -j, -k, or -l may be specified alone\n",
+      fprintf (stderr, "%s: only one list option may be specified\n",
 	       program_name);
       exit (4);
     }
 
-  client_init ();
+  client_init_gnats (user, passwd);
 
   /* Now make our query, and act on what they said.  */
   ask_about (info, argv + optind);
@@ -618,12 +743,13 @@ usage (s)
 {
 #ifdef GNATS_RELEASE_BASED
   fprintf (stderr, "\
-Usage: %s [-DFHhiPqVxE] [-C confidential] [-c category] [-d directory]\n\
+Usage: %s [-DFGHhiIqVxE] [-C confidential] [-c category] [-d directory]\n\
        [-e severity] [-m mtext] [-O originator] [-o outfile] [-p priority]\n\
        [-L class] [-r responsible] [-S submitter] [-s state] [-t text]\n\
-       [-y synopsis] [-A release] [-H host] [-P port] [--full] [--help]\n\
-       [--print-path] [--version] [--summary] [--sql]\n\
-       [--skip-closed] [--category=category]\n\
+       [-b date] [-a date] [-B date] [-M date] [-z date] [-Z date]\n\
+       [-y synopsis] [-A release] [-H host] [-P port] [-v userid] [-w password]\n\
+       [--full] [--help][--print-path] [--version] [--summary]\n\
+       [--sql] [--sql2][--skip-closed] [--category=category]\n\
        [--confidential=yes|no] [--directory=directory] [--output=outfile]\n\
        [--originator=name] [--priority=level] [--class=class]\n\
        [--severity=severity] [--state=state] [--submitter=submitter]\n\
@@ -631,23 +757,31 @@ Usage: %s [-DFHhiPqVxE] [-C confidential] [-c category] [-d directory]\n\
        [--quarter=quarter] [--keywords=regexp]\n\
        [--required-before=date] [--required-after=date]\n\
        [--arrived-before=date] [--arrived-after=date] [--debug]\n\
-       [--list-categories] [--list-responsible] [--list-submitters]\n\
-       [--text=text] [--multitext=mtext] [--host=server] [--port=port] [PR] [PR]...\n",
+       [--modified-before=date] [--modified-after=date]\n\
+       [--closed-before=date] [--closed-after=date]\n\
+       [--list-categories] [--list-classes] [--list-responsible]\n\
+       [--list-states] [--list-submitters] [--list-config]\n\
+       [--text=text] [--multitext=mtext] [--host=server] [--port=port]\n\
+       [--user=userid] [--passwd=password] [PR] [PR]...\n",
 	   program_name);
 #else
   fprintf (stderr, "\
-Usage: %s [-DFHhiPqVxE] [-C confidential] [-c category] [-d directory]\n\
+Usage: %s [-DFGHhiIPqVxE] [-C confidential] [-c category] [-d directory]\n\
        [-e severity] [-m mtext] [-O originator] [-o outfile] [-p priority]\n\
        [-L class] [-r responsible] [-S submitter] [-s state] [-t text]\n\
-       [-y synopsis] [-A release] [-H host] [-P port] [--full] [--help]\n\
-       [--print-path] [--version] [--summary] [--sql]\n\
-       [--skip-closed] [--category=category]\n\
+       [-b date] [-a date] [-B date] [-M date] [-z date] [-Z date]\n\
+       [-y synopsis] [-A release] [-H host] [-P port] [-v userid] [-w password]\n\
+       [--full] [--help] [--print-path] [--version] [--summary]\n\
+       [--sql] [--sql2] [--skip-closed] [--category=category]\n\
        [--confidential=yes|no] [--directory=directory] [--output=outfile]\n\
        [--originator=name] [--priority=level] [--class=class]\n\
        [--severity=severity] [--state=state] [--submitter=submitter]\n\
        [--responsible=person] [--release=release] [--restricted]\n\
        [--arrived-before=date] [--arrived-after=date] [--debug]\n\
-       [--list-categories] [--list-responsible] [--list-submitters]\n\
+       [--modified-before=date] [--modified-after=date]\n\
+       [--closed-before=date] [--closed-after=date]\n\
+       [--list-categories] [--list-classes] [--list-responsible]\n\
+       [--list-states] [--list-submitters] [--list-config]\n\
        [--text=text] [--multitext=mtext] [--host=server] [--port=port] [PR] [PR]...\n",
 	   program_name);
 #endif
