@@ -26,6 +26,12 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #define BY_CATEGORY	2
 #define BY_SUBMITTER	3
 
+/* If 1, we're running the daemon.  */
+int is_daemon = 0;
+
+/* If 1, we're running a network client.  */
+int is_client = 1;
+
 /* The name this program was run with.  */
 char *program_name;
 
@@ -42,6 +48,11 @@ struct option long_options[] =
   {"submitter", 1, NULL, 's'},
   {"category", 1, NULL, 'c'},
   {"debug", 0, NULL, 'D'},
+  {"directory", 1, NULL, 'd'},
+  {"host", 1, NULL, 'H'},
+  {"port", 1, NULL, 'P'},
+  {"user", 1, NULL, 'v'},
+  {"passwd", 1, NULL, 'w'},
   {"help", 0, NULL, 'h'},
   {"version", 0, NULL, 'V'},
   {NULL, 0, NULL, 0},
@@ -88,18 +99,43 @@ main (argc, argv)
   int optc;
   char *arg = NULL;
   int lookups = 0, lookup_type = 0;
+  char *user = "anonymous", *passwd = "guest";
 
+  outfp = stdout;
+  
   program_name = basename (argv[0]);
+  host_name = gnats_server;
+  
   if (argc == 1)
     usage (1);
 
   host_name = gnats_server;
 
-  while ((optc = getopt_long (argc, argv, "p:c:s:VD",
+  while ((optc = getopt_long (argc, argv, "p:c:s:VDd:p:H:v:w:",
 			      long_options, (int *) 0)) != EOF)
     {
       switch (optc)
 	{
+	case 'd':
+	  gnats_root = optarg;
+	  break;
+
+	case 'H':
+	  host_name = optarg;
+	  break;
+
+	case 'P':
+	  port = atoi (optarg);
+	  break;
+
+        case 'v':
+          user = optarg;
+          break;
+
+        case 'w':
+          passwd = optarg;
+          break;
+
 	case 'p':
 	  lookup_type = BY_PR;
 	  lookups++;
@@ -145,7 +181,7 @@ main (argc, argv)
   if (! arg)
     usage (1);
 
-  client_init ();
+  client_init_gnats (user, passwd);
 
   do_lookup (lookup_type, arg);
 
@@ -159,9 +195,11 @@ usage (status)
      int status;
 {
   fprintf (stderr, "\
-Usage: %s [-V] [-p pr] [-s submitter] [-c category] [--version]\n\
-       [--pr=PR] [--submitter=submitter] [--category=category]\n",
-	   program_name);
+Usage: %s [-p|--pr=PR] [-V|--version] [-h|--help] [-D|--debug]\n\
+               [-s|--submitter=submitter] [-c|--category=category]\n\
+               [-d|--directory=dbalias] [-H|--host=host] [-P|--port=port]\n\
+               [-v|--userid=userid] [-w|--password=passwd]\n",
+	program_name);
   exit (status);
 }
 
