@@ -151,7 +151,7 @@ read_pr (fp, prune)
 	     unformatted field can show up anywhere.  */
 	  multi_text = FALSE;
 
-	  if ((len + unformatted_len) >= unformatted_size)
+	  if ((len + unformatted_len + 1) >= unformatted_size)
 	    {
 	      unformatted_size = (unformatted_size * 2) + 1;
 	      unformatted = (char *) xrealloc (unformatted,
@@ -159,8 +159,13 @@ read_pr (fp, prune)
 	      u = unformatted + unformatted_len;
 	    }
 
+	  /* Put a space in front of each unformatted line, showing it as
+	     "quoted text" */
+
+	  *u = ' ';
+	  u++;
 	  memcpy (u, line, len);
-	  unformatted_len += len;
+	  unformatted_len += (len + 1);
 	  u += len;
 	}
     }
@@ -204,7 +209,10 @@ read_pr (fp, prune)
 	  xfree (pr[UNFORMATTED].value);
 	}
 
-      pr[UNFORMATTED].value = unformatted;
+      if (pr[DESCRIPTION].value)
+	pr[UNFORMATTED].value = unformatted;
+      else
+	pr[DESCRIPTION].value = unformatted;
     }
   else
     xfree (unformatted);
@@ -214,6 +222,45 @@ read_pr (fp, prune)
    newline.  Needed for the network portion.  */
 int doret = 0;
 
+/* If TRUE, PR is being written to the network and multitext lines beginning
+   with a dot need to be escaped with another dot */
+int doescape = 0;
+
+/* Escape single dots and end each line with \r\n */
+void
+write_multitext (fp, str)
+     FILE *fp;
+     char *str;
+{
+  char *s, *e, *end = str + strlen (str);
+
+  if (!doescape)
+    {
+      fprintf (fp, "%s", str);
+      return;
+    }
+  
+  s = str;
+  while (s < end)
+    {
+      if (*s == '.')
+        fprintf (fp, ".");
+      e = strchr (s, '\n');
+      if (e)
+        {
+          *e = '\0';
+          fprintf (fp, "%s\r\n", s);
+          *e = '\n';
+          s = e + 1;
+        }
+      else
+        {
+          fprintf (fp, "%s\r\n", s);
+          break;
+        }
+    }
+}
+
 /* Write either a single PR field, or the entire PR into the file passed
    in via FP.  */
 void 
@@ -222,32 +269,40 @@ write_pr (fp, string)
      PR_Name string;
 {
   register int i;
-  extern char *ret;
+  char *ret; /* local, not global */
   int len;
 
 #define MAYBE_NL(str) { \
     len = strlen(str); \
     if (len && (str[len-1] != '\n')) \
-      fputc('\n', fp); \
+      fprintf (fp, "%s", ret); \
   }
 
+  if (doescape)
+    ret = "\r\n";
+  else
+    ret = "\n";
+  
   if (string != NUM_PR_ITEMS)
     {
       char fmt[10];
-      if (pr[string].datatype == MultiText)
-	sprintf (fmt, "%%s%s%%s%s", ret, ret);
-      else
-	sprintf (fmt, "%%-16s %%s%s", ret);
-
+      
       if (pr[string].value == NULL)
 	fprintf (fp, "%s%s", pr[string].name, ret);
       else
 	{
-	  fprintf (fp, fmt, pr[string].name, pr[string].value);
 	  if (pr[string].datatype == MultiText)
 	    {
+              sprintf (fmt, "%%s%s", ret);
+              fprintf (fp, fmt, pr[string].name, pr[string].value);
+              write_multitext (fp, pr[i].value);
 	      MAYBE_NL(pr[string].value);
 	    }
+          else
+            {
+              sprintf (fmt, "%%-16s %%s%s", ret);
+              fprintf (fp, fmt, pr[string].name, pr[string].value);
+            }
 	}
     }
   else
@@ -260,7 +315,8 @@ write_pr (fp, string)
 		it will have the newline we need.  */
 	    if (pr[i].datatype == MultiText)
 	      {
-		fprintf (fp, "%s%s%s", pr[i].name, ret, pr[i].value);
+		fprintf (fp, "%s%s", pr[i].name, ret);
+		write_multitext (fp, pr[i].value);
 		MAYBE_NL(pr[i].value);
 	      }
 	    else
@@ -270,7 +326,241 @@ write_pr (fp, string)
   	  fprintf (fp, "%s%s", pr[i].name, ret);
     }
 }
+
+/* Write either a single PR field from the index */
+void 
+write_pr_from_index (fp, name, value)
+     FILE *fp;
+     PR_Name name;
+     char *value;
+{
+  extern char *ret;
+  char   fmt[10], *t;
+  time_t time;
+
+  sprintf (fmt, "%%-16s %%s%s", ret);
+
+  if (pr[name].datatype == Date)
+    {
+      t = (char *) xmalloc (GNATS_TIME_LENGTH);
+      if ((time = (time_t) atol (value)) == 0)
+        t[0] = '\0';
+      else
+        strftime (t, GNATS_TIME_LENGTH, "%a %b %d %H:%M:%S %Z %Y", localtime (&time));
+      fprintf (fp, fmt, pr[name].name, t);
+      xfree (t);
+    }
+  else
+    fprintf (fp, fmt, pr[name].name, value);
+}
 
+
+/* ****** OBSOLETE ******* Replaced by init_states() and init_classes() in files.c */
+
+char *
+get_pr_enum_field (filename, default_value)
+     char *filename, *default_value;
+{
+  char *path = (char *) xmalloc (PATH_MAX);
+  char *enum_field;
+  /* Init len to something > 4, due to assumptions below. */
+  int len = 0, capacity = 80;
+  int meaningful_line;
+  FILE *fp;
+  char c;
+  
+  sprintf (path, "%s/gnats-adm/%s", gnats_root, filename);
+  fp = fopen (path, "r");
+
+  if (fp == NULL)
+    return default_value;
+  
+  /* else */
+
+  enum_field = xmalloc (capacity);
+  meaningful_line = 0;
+  while ((c = getc (fp)) != EOF)
+    {
+      /* '#' at the beginning of a line is the comment character */
+      if ((meaningful_line == 0) && (c == '#'))
+        while (((c = getc (fp)) != '\n') && (c != EOF))
+          /* eat until end of line */
+          ;
+
+      /* Also ignore all whitespace lines, because it's pointless
+         hassle for the user if we die on them. */
+      if ((meaningful_line == 0) && ((c == ' ') || (c == '\t')))
+        {
+          while (((c = getc (fp)) == ' ')
+                 || (c == '\t'))
+            /* eat until the whitespace is gone */
+            ;
+          if ((c != '\n') && (c != EOF))
+            {
+              punt (0, "unexpected leading whitespace in %s", path);
+              return default_value;
+            }
+        }
+      
+      /* Skip empty lines too. */
+      if ((meaningful_line == 0) && (c == '\n'))
+        continue;
+
+      /* No way can ':' be the first character on a line. */
+      if ((meaningful_line == 0) && (c == ':'))
+        {
+          punt (0, "empty enumerated field name name in %s", path);
+          return default_value;
+        }
+
+      /* There may be a one-line description after the field, but we
+         ignore that right now. */
+      if ((meaningful_line == 1) && (c == ':'))
+        while (((c = getc (fp)) != '\n') && (c != EOF))
+          /* eat until end of line */
+          ;        
+
+      /* Check again -- we might have read EOF during one of the inner
+         `while' loops above. */
+      if (c == EOF)
+        {
+          c = '\n';
+          ungetc (EOF, fp);
+        }
+
+      /* At last admit that we might be onto something. */
+      meaningful_line = 1;
+
+      /* If we've finished reading a field name, delimit it with what
+         the caller expects */
+      if (c == '\n')
+        {
+          if (len >= (capacity - 4))
+            enum_field = xrealloc (enum_field, (capacity *= 2));
+
+          /* caller separates enum_field with " | " */
+          enum_field[len++] = ' ';
+          enum_field[len++] = '|';
+          enum_field[len++] = ' ';
+
+          /* We're done with this line. */
+          meaningful_line = 0;
+        }
+      else if (! (((c >= '0') && (c <= '9'))
+                  || ((c >= 'A') && (c <= 'Z'))
+                  || ((c >= 'a') && (c <= 'z'))
+                  || (c == '-')
+                  || (c == '_')
+                  || (c == '.')))
+        {
+          /* It seems wise to enforce such restrictions starting now,
+           * even though at this time we only have concrete reasons to
+           * prohibit ctrl and 8-bit chars, '/', ':', '|', and '#'. 
+           */
+          punt (0, "illegal character `%c' in field name, in %s", c, path);
+          return default_value;
+        }
+      else /* it's an allowable character for a field name */
+        {
+          if (len >= (capacity - 2))
+            enum_field = xrealloc (enum_field, (capacity *= 2));
+
+          enum_field[len++] = c;
+        }
+    }
+  fclose (fp);
+
+  /* Null terminate only after cleaning trailing garbage. */
+  while ((enum_field[len - 1] == ' ') || (enum_field[len - 1] == '|'))
+    enum_field[--len] = '\0';
+  
+  enum_field[len] = '\0';
+
+  if ((enum_field == NULL) || (enum_field[0] == '\0'))
+    return default_value;
+
+  /* else */
+
+  return enum_field;
+}
+
+/* Get the first value from a pipe separated list.
+ * For example "open | analyzed | ... "  ==> "open"
+ *
+ * result is malloc'd.
+ *
+ * Return "open" if unable to determine first state from list.
+ */
+char *
+get_default_enum_value (list_str, default_value)
+     char *list_str;
+     char *default_value;
+{
+  char *res;
+  char *end;
+  
+  if (list_str == NULL)
+    return default_value;
+
+  /* else */
+
+  end = strchr (list_str, ' ');
+
+  if (end == NULL)
+    return strdup (list_str);
+
+  /* else */
+
+  res = xmalloc ((end - list_str) + 1);
+  strncpy (res, list_str, (end - list_str));
+  res[(end - list_str)] = '\0';
+
+  return res;
+}
+
+/* Get the last value from a pipe separated list.
+ * For example "... | suspended | closed"  ==> "closed"
+ *
+ * result is malloc'd.
+ *
+ * Return default if unable to determine final state from list.
+ */
+char *
+get_final_enum_value (list_str, default_value)
+     char *list_str;
+     char *default_value;
+{
+  char *res = NULL;
+  char *tmp, *end;
+  
+  if (list_str == NULL)
+    return default_value;
+
+  res = tmp = list_str;
+  while ((tmp != NULL) && ((tmp = strchr (tmp, '|')) != NULL))
+    {
+      tmp++;
+      res = tmp;
+    }
+
+  if ((res == NULL) || (res[0] == '\0'))
+    return default_value;
+
+  /* else */
+
+  /* inch past the space */
+  if (res[0] == ' ')
+    res++;
+
+  if (res[0] == '\0')
+    return default_value;
+
+  /* else */
+
+  return strdup (res);
+}
+
+
 /* Init all the fields in the PR.  */
 void 
 init_pr ()
@@ -300,17 +590,17 @@ init_pr ()
   pr[CONFIDENTIAL].datatype = Enum;
 
   pr[SEVERITY].name = SEVERITY_STRING;
-  pr[SEVERITY].enum_values = "non-critical | serious | critical";
+  pr[SEVERITY].enum_values = "critical | serious | non-critical";
   pr[SEVERITY].default_value = "serious";
   pr[SEVERITY].datatype = Enum;
 
   pr[PRIORITY].name = PRIORITY_STRING;
-  pr[PRIORITY].enum_values = "low | medium | high";
+  pr[PRIORITY].enum_values = "high | medium | low";
   pr[PRIORITY].default_value = "medium";
   pr[PRIORITY].datatype = Enum;
 
   pr[CATEGORY].name = CATEGORY_STRING;
-  pr[CATEGORY].default_value = PENDING;
+  pr[CATEGORY].default_value = default_category;
   pr[CATEGORY].datatype = Text;	/* really enum, matching category file */
 
 #ifdef GNATS_RELEASE_BASED
@@ -319,18 +609,19 @@ init_pr ()
 
   pr[KEYWORDS].name = KEYWORDS_STRING;
   pr[KEYWORDS].datatype = Text;
+
+  pr[DATE_REQUIRED].name = DATE_REQUIRED_STRING;
+  pr[DATE_REQUIRED].datatype = Date;
 #endif
 
   pr[RELEASE].name = RELEASE_STRING;
   pr[RELEASE].datatype = Text;
 
   pr[LAST_MODIFIED].name = LAST_MODIFIED_STRING;
-  pr[LAST_MODIFIED].datatype = Text;
+  pr[LAST_MODIFIED].datatype = Date;
 
-#ifdef GNATS_RELEASE_BASED
-  pr[DATE_REQUIRED].name = DATE_REQUIRED_STRING;
-  pr[DATE_REQUIRED].datatype = Date;
-#endif
+  pr[CLOSED_DATE].name = CLOSED_DATE_STRING;
+  pr[CLOSED_DATE].datatype = Date;
 
   pr[ENVIRONMENT].name = ENVIRONMENT_STRING;
   pr[ENVIRONMENT].datatype = MultiText;
@@ -356,20 +647,30 @@ State-Changed-When:\n\
 State-Changed-Why:";
 
   pr[STATE].name = STATE_STRING;
-  pr[STATE].enum_values = "open | analyzed | feedback | suspended | closed";
-  pr[STATE].default_value = "open";
   pr[STATE].datatype = Enum;
+  if (! is_client)
+    {
+      init_states ();
+      pr[STATE].enum_values = get_state_enum_field ();
+      pr[STATE].default_value = get_default_enum_value (pr[STATE].enum_values, "open");
+    }
 
   pr[CLASS].name = CLASS_STRING;
-  pr[CLASS].enum_values
-    = "support | sw-bug | doc-bug | change-request | mistaken | duplicate ";
-  pr[CLASS].default_value = "sw-bug";
   pr[CLASS].datatype = Enum;
+  if (! is_client)
+    {
+      init_classes ();
+      pr[CLASS].enum_values = get_class_enum_field ();
+      pr[CLASS].default_value = get_default_enum_value (pr[CLASS].enum_values, "sw-bug");
+    }
 
   pr[FIX].name = FIX_STRING;
   pr[FIX].datatype = MultiText;
   pr[FIX].default_value = "\nUnknown";
 
+  pr[RELEASE_NOTE].name = RELEASE_NOTE_STRING;
+  pr[RELEASE_NOTE].datatype = MultiText;
+
   pr[UNFORMATTED].name = UNFORMATTED_STRING;
   pr[UNFORMATTED].datatype = MultiText;
 }
@@ -468,7 +769,12 @@ check_enum_types (check)
 			     "\tNote: There was a bad value `%s' for the field `%s'.\n\tIt was set to the default value of `%s'.\n",
 			     pr[i].value, pr[i].name, pr[i].default_value);
 		  else if (check == 2)
-		    sprintf (msg, "%s %s\n", pr[i].name, pr[i].value);
+                    {
+                      if (is_daemon)
+		        sprintf (msg, "%s %s", pr[i].name, pr[i].value);
+                      else
+		        sprintf (msg, "%s %s\n", pr[i].name, pr[i].value);
+                    }
 
 		  bad_enums_end->msg = msg;
 		  bad_enums_end->next = (struct bad_enum *) NULL;
