@@ -75,6 +75,6 @@ void
 xfree (p)
      char *p;
 {
-  if (p != 0)
+  if (p != NULL)
     free (p);
 }
