@@ -59,14 +59,6 @@ MAIL_AGENT="xMAIL_AGENTx"
 
 # How to read the passwd database.
 PASSWD="xPASSWDx"
-PASSWD="cat /etc/passwd"
-if [ -x /bin/domainname ]; then
-  if [ -x /usr/bin/niscat ]; then
-    PASSWD="/usr/bin/niscat passwd.org_dir"
-  elif [ -x /usr/bin/ypcat ]; then
-    PASSWD="/usr/bin/ypcat passwd"
-  fi
-fi
 
 ECHON=xECHONx
 
@@ -105,9 +97,7 @@ if [ "$LOGNAME" = "" ]; then
 fi
 
 FROM="$LOGNAME"
-if [ -z "$REPLYTO" ]; then
-  REPLYTO="$LOGNAME"
-fi
+REPLY_TO="${REPLY_TO:-${REPLYTO:-$LOGNAME}}"
 
 # Find out the name of the originator of this PR.
 if [ -n "$NAME" ]; then
@@ -131,8 +121,6 @@ else
     ORGANIZATION="$DEFAULT_ORGANIZATION"
   elif [ -f $HOME/.organization ]; then
     ORGANIZATION="`cat $HOME/.organization`"
-  elif [ -f $HOME/.signature ]; then
-    ORGANIZATION="`cat $HOME/.signature`"
   fi
 fi
 
@@ -264,7 +252,7 @@ if [ -z "$SEVERITY_C" ]; then
   SEVERITY_C='<[ non-critical | serious | critical ] (one line)>'
 fi
 PRIORITY_C='<[ low | medium | high ] (one line)>'
-CATEGORY_C='<name of the product (one line)>'
+CATEGORY_C='<choose from the list of categories above (one line)>'
 CLASS_C='<[ sw-bug | doc-bug | change-request | support ] (one line)>'
 RELEASE_C='<release number or tag (one line)>'
 ENVIRONMENT_C='<machine, os, target, libraries (multiple lines)>'
@@ -317,6 +305,9 @@ SEND-PR: will all comments (text enclosed in `<' and `>').
 SEND-PR: 
 SEND-PR: Please consult the send-pr man page `send-pr(1)' or the Texinfo
 SEND-PR: manual if you are not sure how to fill out a problem report.
+SEND-PR: Note that the Synopsis field is mandatory.  The Subject (for
+SEND-PR: the mail) will be made the same as Synopsis unless explicitly
+SEND-PR: changed.
 SEND-PR:
 SEND-PR: Choose from the following categories:
 SEND-PR:
@@ -341,12 +332,12 @@ From: $FROM
 Reply-To: $REPLYTO
 Cc: $CC
 X-send-pr-version: $VERSION
+X-GNATS-Notify: 
 
 
 >Submitter-Id:	$SUBMITTER
 >Originator:	$ORIGINATOR
->Organization:
-${ORGANIZATION-	$ORGANIZATION_C}
+>Organization:	${ORGANIZATION-$ORGANIZATION_C}
 >Confidential:	$CONFIDENTIAL_C
 >Synopsis:	$SYNOPSIS_C
 >Severity:	$SEVERITY_C
@@ -355,10 +346,10 @@ ${ORGANIZATION-	$ORGANIZATION_C}
 >Class:		$CLASS_C
 >Release:	${DEFAULT_RELEASE-$RELEASE_C}
 >Environment:
-	$ENVIRONMENT_C
 `[ -n "$SYSTEM" ] && echo System: $SYSTEM`
 `[ -n "$ARCH" ] && echo Architecture: $ARCH`
 `[ -n "$MACHINE" ] && echo Machine: $MACHINE`
+	$ENVIRONMENT_C
 >Description:
 	$DESCRIPTION_C
 >How-To-Repeat:
@@ -466,8 +457,17 @@ while [ -z "$REQUEST_ID" ]; do
     ""|sw-bug|doc-bug|change-request|support) CNT=`expr $CNT + 1` ;;
     *)  echo "$COMMAND: \`$CLASS' is not a valid value for \`Class'."
   esac
+  #
+  # 6) Check that synopsis is not empty
+  #
+  if grep "^>Synopsis:[ 	]*${SYNOPSIS_C}\$" $TEMP > /dev/null
+  then
+    echo "$COMMAND: Synopsis must not be empty."
+  else
+    CNT=`expr $CNT + 1`
+  fi
 
-  [ $CNT -lt 5 -a -z "$BATCH" ] && 
+  [ $CNT -lt 6 -a -z "$BATCH" ] && 
     echo "Errors were found with the problem report."
 
   while true; do
@@ -475,7 +475,7 @@ while [ -z "$REQUEST_ID" ]; do
       $ECHON1 "a)bort, e)dit or s)end? $ECHON2"
       read input
     else
-      if [ $CNT -eq 5 ]; then
+      if [ $CNT -eq 6 ]; then
         input=s
       else
         input=a
@@ -501,6 +501,22 @@ while [ -z "$REQUEST_ID" ]; do
     esac
   done
 done
+
+#
+# Make sure the mail has got a Subject.  If not, use the same as
+# in Synopsis.
+#
+
+if grep '^Subject:[ 	]*$' $TEMP > /dev/null
+then
+  SYNOPSIS=`grep '^>Synopsis:' $TEMP | sed -e 's/^>Synopsis:[ 	]*//'`
+  ed -s $TEMP << __EOF__
+/^Subject:/s/:.*\$/: $SYNOPSIS/
+w
+q
+__EOF__
+fi
+
 #
 #	Remove comments and send the problem report
 #	(we have to use patterns, where the comment contains regex chars)
